/**
 * COPYRIGHT 2023 Flirc, Inc. All rights reserved.
 *
 * This copyright notice is Copyright Management Information under 17 USC 1202
 * and is included to protect this work and deter copyright infringement.
 * Removal or alteration of this Copyright Management Information without
 * the express written permission of Flirc, Inc. is prohibited, and any
 * such unauthorized removal or alteration will be a violation of federal law.
 */

#ifndef I__MMAP_H__
	#define I__MMAP_H__

#ifdef __cplusplus
extern "C" {
#endif /* __cpluspuls */

#include <prjutil.h>

/* VENDOR-Flirc Device Requests */
/* bRequest */
#define NORMAL				0x00
#define RECORD				0x01
#define DELETE_KEY			0x02
#define FORMAT				0x03
#define DELETE_KEYS			0x04
#define FLIRC_STATUS			0x05
#define CLEAR_STATUS			0x06
#define SUSPEND				0x07 /* Enable Disable */

/**
 * VENDOR Flirc Endpoint Requests
 *
 * bRequests
 */
#define GET_VERSION			0x01
#define PEEK				0x02
#define POKE				0x03
#define GET_BOOT_VERSION		0x04
#define UPGRADE				0x05
#define REBOOT				0x06
#define UPDATE_CONFIGS			0x07
#define UPGRADE_NEW			0x08
/* bRequests -- Bootloader Specific */
#define GET_DEV_INFO			0x01
#define GET_PAGE_STATUS			0x08
#define SEND_PAGE			0x09
#define CHECK_STATUS			0x0A
#define LEAVE_BOOTLDR			0x0B
#define GET_SCMHASH			0x0C
#define GET_SKU				0x0D

/* Define the significant memory locations, see Documentation Below */
#define loc_used_space_h		0
#define loc_used_space_l		1
#define loc_of_settings 		2
#define loc_of_settings2		3
#define loc_DFU				4
#define loc_of_first_hash		5

struct key_t {
	uint32_t hash;
	uint16_t key_report;
} __packed;;

struct mem_t {
	uint16_t size;
	uint8_t settings;
	uint8_t settings2;
	uint8_t dfu;
	uint8_t *buf;
} __packed;


/* SETTINGS */
#define BOOTLDR_STATUS 			1:7 /* 1 = error, 0 = ok */
#define RESERVED			1:6 /* 0 = default, 1 = unknown */
#define SENSITIVITY			2:4 /* see ir_transceiver.c */
#define INTERKEY_DELAY			4:0 /* see ir_transceiver */

/* SETTINGS 2 */
#define SUSPEND_SUPPORT			1:7 /* 1 = on,  0 = off */
#define NOISE_CANCEL			1:6 /* 1 = on,  0 = off */
#define BUILTIN_PROFILES		1:5 /* 1 = off, 0 = on  */
#define SEQUENCE_MODIFIERS		1:4 /* 1 = off, 0 = on  */
#define DEVICE_ID			2:1 /* See table below  */
#define UNIT_TESTED			1:0 /* 1 =yes,  1 = no  */
#define DEFAULT_SUSPEND_SUPPORT		1
#define DEFAULT_NOISE_CANCEL		0
#define DEFAULT_BUILTIN_PROFILES	0
#define DEF_SEQUENCE_MODIFIERS		1

/* Device ID Table
 * 0b11 - Flirc
 * 0b10 - Amazon
 */
#define DEVICE_ID_AMAZON		2
#define DEVICE_ID_FLIRC			3

/* setting definitions */
#define DEFAULT_SENSITIVITY		13
#define BTLDR_TIMER_EN			0x0
#define DEFAULT_INTERKEY_DELAY		0x5

/* USB_mode */
#define USB_MODE 			7:0 /* not implimented */

#define FIRST_BOOT                      0x50
#define STAY_DFU                        0x55
#define DONT_STAY_DFU                   0x05

#ifndef SPM_PAGESIZE
#define SPM_PAGESIZE 128
#endif

#define FL_PAGE_SIZE  SPM_PAGESIZE
#define BTLDR_SIZE    2048

struct dev_info {
	uint16_t page_size;
	uint32_t flash_size;
} __attribute__((packed));

struct dev_info_bl_v1 {
	uint8_t report_id;
	uint16_t page_size;
	uint32_t flash_size;
} __attribute__((packed));

struct packet {
	uint8_t  buf[SPM_PAGESIZE];
	uint16_t page_index;
	uint8_t  crc;
} __attribute__((packed));

struct packet_bl_v1 {
	uint32_t dummy;
	uint8_t buf[SPM_PAGESIZE];
} __attribute__((packed));

struct packet_res {
	uint32_t page_index;
	uint8_t  crc_err;
} __attribute__((packed));

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* I__MMAP_H__ */

